// File : 'Squirrel.h'
// Created by FishIDE application 
#ifndef __FISHINOSQUIRREL_SERVER_H
#define __FISHINOSQUIRREL_SERVER_H

#include <FishinoWebServer32.h>
#include <FishinoSquirrel.h>

#include "StringStream.h"

//////////////////////////////////////////////////////////////////////////////////////////
// some detaisl on filesystem on SD card
#define SQUIRREL_PORT		8080
#define SQUIRREL_ROOT		"/SQUIRREL"
#define PROJECTS_ROOT		SQUIRREL_ROOT "/PROJECTS"
#define LIBS_ROOT			SQUIRREL_ROOT "/LIBRARIES"
#define SQUIRREL_INDEX		"/index.html"
//////////////////////////////////////////////////////////////////////////////////////////

#define SQSERVER FishinoSquirrelServerClass::__getFishinoSquirrelServer()

class FishinoSquirrelServerClass
{
	public:
		static FishinoSquirrelServerClass &__getFishinoSquirrelServer() ;
	private:

		// static functions and callbacks
		static bool __fileHandler(FishinoWebServer &) ;
		static bool __indexHandler(FishinoWebServer &) ;
		static bool __sqfishHandler(FishinoWebServer &) ;
		static bool __fileUploaderHandler(FishinoWebServer &, uint8_t action, char* buffer, int size) ;
		static bool __redirectHandler(FishinoWebServer &server) ;
	
		// the web server object
		FishinoWebServer _server;
		
		// the squirrel object
		FishinoSquirrel _sq;

		// constructor -- don't allow construction
		FishinoSquirrelServerClass() ;
		
		// post data parsed by parsePostData function
		uint16_t _postCount;
		char *_postBuf;
		char **_postNames;
		char **_postValues;
		
		// terminal messages queue
		StringStream _terminalStream;
		
		// file object - used for uploads
		SdFile *_uploadFile;

	protected:
	
		// clear the post data
		void clearPostData(void) ;
	
		// parse post data from client in form of url-encoded data
		// PATH=PROJECTS%2F&COMMAND=LIST&FOLDERS=true
		// builds 2 arrays, one holding names and the other the values
		bool parsePostData(void) ;
		
		// get post value from name
		const char *getPostValue(const char *name) const ;
		
		// build full path from PATH post value
		String getFullPath(void) ;

		// sends a file to client
		void sendFile(const char* filename) ;
		
		// handle file requests
		bool fileHandler(void) ;

		// handle index.htm requests
		bool indexHandler(void) ;

		// handle sqfish.php requests
		bool sqfishHandler(void) ;

		// handle file uploads
		bool fileUploaderHandler(uint8_t action, char* buffer, int size) ;
		
		// load execution manager and run it
		bool runApplication(void) ;

		// run a project
		bool runProject(const char *name) ;

	public:

		// destructor
		~FishinoSquirrelServerClass() ;
		
		void begin(void) ;
		void end() ;
		
		// run the last session's project name
		bool runLastProject(void);
		
		// main loop 
		void loop(void) ;
		
		// output function
		void puts(const char *s) ;
};

#endif
